@@ -1,39 +1,55 @@
-/* vim: set expandtab ts=8 sw=4: */
-
-/*  $Id: panel-app.c 28348 2008-10-21 20:12:41Z olivier $
+/* $Id: panel-app.c 28386 2008-10-23 17:35:49Z nick $
  *
- *  Copyright © 2005-2006 Jasper Huijsmans <jasper@xfce.org>
+ * Copyright (c) 2005 Jasper Huijsmans <jasper@xfce.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published 
- *  by the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published
+ * by the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <stdio.h> 
+#ifdef HAVE_STDIO_H
+#include <stdio.h>
+#endif
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
+#endif
+#ifdef HAVE_SIGNAL_H
 #include <signal.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
+#ifdef HAVE_ERRNO_H
 #include <errno.h>
+#endif
+#ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
+#endif
 
 #include <X11/Xlib.h>
+#include <X11/Xatom.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkx.h>
 #include <libxfcegui4/libxfcegui4.h>
 #include <libxfce4panel/xfce-panel-macros.h>
 #include <libxfce4panel/xfce-panel-convenience.h>
@@ -60,20 +76,20 @@
 
 #if defined(TIMER) && defined(G_HAVE_ISO_VARARGS)
 void
-xfce_panel_program_log (const char *file, 
-                        const int   line, 
-                        const char *format, 
+xfce_panel_program_log (const gchar *file,
+                        const gint   line,
+                        const gchar *format,
                         ...)
 {
-    va_list args;
-    gchar  *formatted;
-    gchar  *message;
+    va_list  args;
+    gchar   *formatted;
+    gchar   *message;
 
     va_start (args, format);
     formatted = g_strdup_vprintf (format, args);
     va_end (args);
 
-    message = g_strdup_printf ("MARK: %s: %s:%d: %s", 
+    message = g_strdup_printf ("MARK: %s: %s:%d: %s",
                                g_get_prgname(), file, line, formatted);
 
     access (message, F_OK);
@@ -107,20 +123,21 @@ struct _PanelApp
     GPtrArray     *panel_list;
     GPtrArray     *monitor_list;
 
-    int            save_id;
-    int            current_panel;
+    gint           save_id;
+    gint           current_panel;
+
     GList         *dialogs;
 
     /* Initialization. Also unset before cleanup. */
-    guint          initialized:1; 
+    guint          initialized : 1;
 
     /* Check whether monitors in Xinerama are aligned. */
-    guint          xinerama_and_equal_width:1;
-    guint          xinerama_and_equal_height:1;
+    guint          xinerama_and_equal_width : 1;
+    guint          xinerama_and_equal_height : 1;
 };
 
 static PanelApp panel_app = {0};
-static int signal_pipe[2];
+static gint     signal_pipe[2];
 
 
 /* cleanup */
@@ -128,8 +145,10 @@ static int signal_pipe[2];
 static void
 cleanup_panels (void)
 {
-    int    i;
-    GList *l;
+    guint        i;
+    GList       *l;
+    Panel       *panel;
+    XfceMonitor *xmon;
 
     if (!panel_app.initialized)
         return;
@@ -138,7 +157,7 @@ cleanup_panels (void)
 
     l = panel_app.dialogs;
     panel_app.dialogs = NULL;
-    
+
     while (l)
     {
         gtk_dialog_response (GTK_DIALOG (l->data), GTK_RESPONSE_NONE);
@@ -147,7 +166,7 @@ cleanup_panels (void)
 
     for (i = 0; i < panel_app.panel_list->len; ++i)
     {
-        Panel *panel = g_ptr_array_index (panel_app.panel_list, i);
+        panel = g_ptr_array_index (panel_app.panel_list, i);
 
         gtk_widget_hide (GTK_WIDGET (panel));
 
@@ -162,19 +181,20 @@ cleanup_panels (void)
 
     for (i = 0; i < panel_app.monitor_list->len; ++i)
     {
-        XfceMonitor *xmon = g_ptr_array_index (panel_app.monitor_list, i);
+        xmon = g_ptr_array_index (panel_app.monitor_list, i);
         panel_slice_free (XfceMonitor, xmon);
     }
     g_ptr_array_free (panel_app.monitor_list, TRUE);
 }
 
+
 /* signal handling */
 
 /** copied from glibc manual, does this prevent zombies? */
 static void
-sigchld_handler (int sig)
+sigchld_handler (gint sig)
 {
-    int pid, status, serrno;
+    gint pid, status, serrno;
 
     serrno = errno;
 
@@ -190,12 +210,12 @@ sigchld_handler (int sig)
 }
 
 static void
-sighandler (int sig)
+sighandler (gint sig)
 {
     /* Don't do any real stuff here.
-     * Only set a signal state flag. There's a timeout in the main loop
-     * that tests the flag.
-     * This will prevent problems with gtk main loop threads and stuff
+     * Only write the signal to a pipe. The value will be picked up by a
+     * g_io_channel watch.  This will prevent problems with gtk main loop 
+     * threads and stuff.
      */
     if (write (signal_pipe[1], &sig, sizeof (int)) != sizeof (int))
     {
@@ -204,18 +224,18 @@ sighandler (int sig)
 }
 
 static gboolean
-check_run_state (void)
+evaluate_run_state (void)
 {
-    static int recursive = 0;
-    gboolean   quit      = FALSE;
- 
+    static gint recursive = 0;
+    gboolean    quit = FALSE;
+
     /* micro-optimization */
     if (G_LIKELY (panel_app.runstate == PANEL_RUN_STATE_NORMAL))
         return TRUE;
-    
+
     if (G_UNLIKELY (recursive))
         return TRUE;
-    
+
     recursive++;
 
     switch (panel_app.runstate)
@@ -241,7 +261,7 @@ check_run_state (void)
                 panel_app_save ();
                 quit = TRUE;
             }
-            
+
             /* don't forget to set back the run state */
             panel_app.runstate = PANEL_RUN_STATE_NORMAL;
             break;
@@ -249,13 +269,21 @@ check_run_state (void)
 
     if (quit)
     {
+        /* we quit on purpose, update session manager so 
+         * it does not restart the program immediately */
+        client_session_set_restart_style(panel_app.session_client, SESSION_RESTART_IF_RUNNING);
+        
         if (panel_app.save_id)
+        {
             g_source_remove (panel_app.save_id);
-        panel_app.save_id = 0;
+            panel_app.save_id = 0;
+        }
+
+        close (signal_pipe[0]);
+        close (signal_pipe[1]);
 
-        /* this is necessary, because the function is not only called from the
-         * timeout, so just returning FALSE is not enough. */
         gtk_main_quit ();
+
         return FALSE;
     }
 
@@ -265,37 +293,19 @@ check_run_state (void)
 }
 
 static gboolean
-set_run_state (GIOChannel   *source, 
-               GIOCondition  cond, 
-               gpointer      data)
+signal_pipe_io (GIOChannel   *source,
+                GIOCondition  cond,
+                gpointer      data)
 {
-    GError    *error = NULL;
-    GIOStatus  status;
-    gsize      bytes_read;
-    /* 
-     * There is no g_io_channel_read or g_io_channel_read_int, so we read
-     * char's and use a union to recover the unix signal number.
-     */
-    union
-    {
-        gchar chars[sizeof (int)];
-        int   signal;
-    } buf;
+    gint  signal;
+    gsize bytes_read;
 
-    while ((status = g_io_channel_read_chars (source, buf.chars, sizeof (int),
-                                              &bytes_read, &error)
-           ) == G_IO_STATUS_NORMAL)
+    if (G_IO_STATUS_NORMAL == g_io_channel_read_chars (source, (gchar *)&signal, 
+                                                       sizeof (signal),
+                                                       &bytes_read, NULL)
+        && sizeof(signal) == bytes_read)
     {
-        if (bytes_read != sizeof (int))
-        {
-            g_printerr ("lost data in signal pipe: expected %d, receieved %d",
-                        sizeof (int), bytes_read);
-            /* always at least quit if we received data */
-            panel_app.runstate = PANEL_RUN_STATE_QUIT_NOCONFIRM;
-            continue;
-        }
-
-        switch (buf.signal)
+        switch (signal)
         {
             case SIGUSR1:
                 DBG ("USR1 signal caught");
@@ -319,28 +329,86 @@ set_run_state (GIOChannel   *source,
         }
     }
 
-    if (error != NULL)
+    return evaluate_run_state ();
+}
+
+static gboolean
+init_signal_pipe ( void )
+{
+    GIOChannel *g_signal_in;
+    glong       fd_flags;
+
+    /* create pipe and set writing end in non-blocking mode */
+    if (pipe (signal_pipe))
     {
-        g_printerr ("reading signal pipe failed: %s\n", error->message);
-        g_error_free (error);
-        panel_app.runstate = PANEL_RUN_STATE_QUIT_NOCONFIRM;
-    }
-    if (status != G_IO_STATUS_AGAIN)
-    {
-        g_printerr ("signal pipe has been closed\n");
-        panel_app.runstate = PANEL_RUN_STATE_QUIT_NOCONFIRM;
+        return FALSE;
     }
 
-    return check_run_state ();
+    fd_flags = fcntl (signal_pipe[1], F_GETFL);
+    if (fd_flags == -1)
+    {
+        return FALSE;
+    }
+
+    if (fcntl (signal_pipe[1], F_SETFL, fd_flags | O_NONBLOCK) == -1)
+    {
+        close (signal_pipe[0]);
+        close (signal_pipe[1]);
+
+        return FALSE;
+    }
+
+    /* convert the reading end of the pipe into a GIOChannel */
+    g_signal_in = g_io_channel_unix_new (signal_pipe[0]);
+    g_io_channel_set_encoding (g_signal_in, NULL, NULL);
+    g_io_channel_set_close_on_unref (g_signal_in, FALSE);
+
+    /* register the reading end with the event loop */
+    g_io_add_watch (g_signal_in, G_IO_IN | G_IO_PRI, signal_pipe_io, NULL);
+    g_io_channel_unref (g_signal_in);
+
+    return TRUE;
+}
+
+static void
+init_signal_handlers ( void )
+{
+#ifdef HAVE_SIGACTION
+    struct sigaction  act;
+
+    act.sa_handler = sighandler;
+    sigemptyset (&act.sa_mask);
+# ifdef SA_RESTART
+    act.sa_flags = SA_RESTART;
+# else
+    act.sa_flags = 0;
+# endif
+    sigaction (SIGHUP, &act, NULL);
+    sigaction (SIGUSR1, &act, NULL);
+    sigaction (SIGUSR2, &act, NULL);
+    sigaction (SIGINT, &act, NULL);
+    sigaction (SIGABRT, &act, NULL);
+    sigaction (SIGTERM, &act, NULL);
+    act.sa_handler = sigchld_handler;
+    sigaction (SIGCHLD, &act, NULL);
+#else
+    signal (SIGHUP, sighandler);
+    signal (SIGUSR1, sighandler);
+    signal (SIGUSR2, sighandler);
+    signal (SIGINT, sighandler);
+    signal (SIGABRT, sighandler);
+    signal (SIGTERM, sighandler);
+    signal (SIGCHLD, sigchld_handler);
+#endif
 }
 
 /* session */
 
 static void
-session_save_yourself (gpointer data, 
-                       int      save_style, 
+session_save_yourself (gpointer data,
+                       gint     save_style,
                        gboolean shutdown,
-                       int      interact_style, 
+                       gint     interact_style,
                        gboolean fast)
 {
     panel_app_save ();
@@ -356,7 +424,7 @@ session_die (gpointer client_data)
 static void
 monitor_size_changed (GdkScreen *screen)
 {
-    int          i;
+    guint        i;
     XfceMonitor *monitor;
     GtkWidget   *panel;
 
@@ -370,7 +438,7 @@ monitor_size_changed (GdkScreen *screen)
          */
         if ((monitor->screen == screen) && (monitor->num < gdk_screen_get_n_monitors (screen)))
         {
-            gdk_screen_get_monitor_geometry (screen, monitor->num, 
+            gdk_screen_get_monitor_geometry (screen, monitor->num,
                                              &(monitor->geometry));
         }
     }
@@ -390,12 +458,14 @@ create_monitor_list (void)
     GdkDisplay  *display;
     GdkScreen   *screen;
     XfceMonitor *monitor;
-    int          n_screens;
-    int          n_monitors = 0;
-    int          i, j;
-    int          w = 0, h = 0;
+    gint         n_screens;
+    gint         n_monitors = 0;
+    gint         i, j;
+    guint        k, m;
+    gint         w = 0, h = 0;
     gboolean     equal_w, equal_h;
-    
+    XfceMonitor *mon1, *mon2;
+
     panel_app.monitor_list = g_ptr_array_new ();
 
     display = gdk_display_get_default ();
@@ -403,7 +473,7 @@ create_monitor_list (void)
     n_screens = gdk_display_get_n_screens (display);
 
     equal_w = equal_h = TRUE;
-    
+
     for (i = 0; i < n_screens; ++i)
     {
         screen = gdk_display_get_screen (display, i);
@@ -420,7 +490,7 @@ create_monitor_list (void)
             gdk_screen_get_monitor_geometry (screen, j, &(monitor->geometry));
 
             g_ptr_array_add (panel_app.monitor_list, monitor);
-            
+
             if (j > 0)
             {
                 if (w != monitor->geometry.width)
@@ -431,7 +501,7 @@ create_monitor_list (void)
 
             w = monitor->geometry.width;
             h = monitor->geometry.height;
-            
+
 #if TEST_MULTIPLE_MONITORS
             monitor = panel_slice_new0 (XfceMonitor);
 
@@ -444,7 +514,7 @@ create_monitor_list (void)
 #endif
         }
 
-        g_signal_connect (screen, "size-changed", 
+        g_signal_connect (G_OBJECT (screen), "size-changed",
                           G_CALLBACK (monitor_size_changed), NULL);
     }
 
@@ -455,18 +525,16 @@ create_monitor_list (void)
     }
 
     /* check layout */
-    for (i = 0; i < panel_app.monitor_list->len; ++i)
+    for (k = 0; k < panel_app.monitor_list->len; ++k)
     {
-        XfceMonitor *mon1, *mon2;
+        mon1 = g_ptr_array_index (panel_app.monitor_list, k);
 
-        mon1 = g_ptr_array_index (panel_app.monitor_list, i);
-
-        for (j = 0; j < panel_app.monitor_list->len; ++j)
+        for (m = 0; m < panel_app.monitor_list->len; ++m)
         {
-            if (j == i)
+            if (m == k)
                 continue;
 
-            mon2 = g_ptr_array_index (panel_app.monitor_list, j);
+            mon2 = g_ptr_array_index (panel_app.monitor_list, m);
 
             if (mon2->geometry.x < mon1->geometry.x
                 && mon2->geometry.y < mon1->geometry.y + mon1->geometry.height
@@ -527,49 +595,49 @@ unregister_dialog (GtkWidget *dialog)
  * Initialize application. Creates ipc window if no other instance is
  * running or sets the ipc window from the running instance.
  *
- * Returns: 0 on success, 1 when an xfce4-panel instance already exists, 
- *          and -1 on failure.
+ * Returns: INIT_SUCCESS (0) on success, INIT_RUNNING (1) when an xfce4-panel 
+ *          instance already exists, and INIT_FAILURE (2) on failure.
  **/
 int
 panel_app_init (void)
 {
-    Atom                 selection_atom;
-    Atom                 manager_atom;
+    Atom                 selection_atom, manager_atom;
     GtkWidget           *invisible;
     XClientMessageEvent  xev;
 
     if (panel_app.initialized)
-        return 0;
-    
+        return INIT_SUCCESS;
+
     panel_app.initialized = TRUE;
-    
+
     selection_atom = XInternAtom (GDK_DISPLAY (), SELECTION_NAME, False);
+
     panel_app.ipc_window = XGetSelectionOwner (GDK_DISPLAY (), selection_atom);
 
     if (panel_app.ipc_window)
-        return 1;
+        return INIT_RUNNING;
 
     invisible = gtk_invisible_new ();
     gtk_widget_realize (invisible);
-    
+
     panel_app.gtk_ipc_window = invisible;
     panel_app.ipc_window = GDK_WINDOW_XWINDOW (invisible->window);
-    
+
     XSelectInput (GDK_DISPLAY (), panel_app.ipc_window, PropertyChangeMask);
-    
-    XSetSelectionOwner (GDK_DISPLAY (), selection_atom, panel_app.ipc_window, 
+
+    XSetSelectionOwner (GDK_DISPLAY (), selection_atom, panel_app.ipc_window,
                         GDK_CURRENT_TIME);
-    
-    if (XGetSelectionOwner (GDK_DISPLAY (), selection_atom) != 
+
+    if (XGetSelectionOwner (GDK_DISPLAY (), selection_atom) !=
             panel_app.ipc_window)
     {
         g_critical ("Could not set ownership of selection \"%s\"",
                     SELECTION_NAME);
-        return -1;
+        return INIT_FAILURE;
     }
-    
+
     manager_atom = XInternAtom (GDK_DISPLAY (), "MANAGER", False);
-    
+
     xev.type         = ClientMessage;
     xev.window       = GDK_ROOT_WINDOW ();
     xev.message_type = manager_atom;
@@ -577,8 +645,8 @@ panel_app_init (void)
     xev.data.l[0]    = GDK_CURRENT_TIME;
     xev.data.l[1]    = selection_atom;
     xev.data.l[2]    = panel_app.ipc_window;
-    xev.data.l[3]    = 0;	/* manager specific data */
-    xev.data.l[4]    = 0;	/* manager specific data */
+    xev.data.l[3]    = 0;    /* manager specific data */
+    xev.data.l[4]    = 0;    /* manager specific data */
 
     XSendEvent (GDK_DISPLAY (), GDK_ROOT_WINDOW (), False,
                 StructureNotifyMask, (XEvent *) & xev);
@@ -586,7 +654,7 @@ panel_app_init (void)
     /* listen for client messages */
     panel_app_listen (invisible);
 
-    return 0;
+    return INIT_SUCCESS;
 }
 
 /* fix position after showing panel for the first time */
@@ -600,161 +668,115 @@ expose_timeout (GtkWidget *panel)
 static void
 panel_app_init_panel (GtkWidget *panel)
 {
-    TIMER_ELAPSED("start panel_app_init_panel: %p", panel);
+    MARK("start panel_app_init_panel: %p", panel);
     panel_init_position (PANEL (panel));
     panel_init_signals (PANEL (panel));
-    TIMER_ELAPSED(" + start show panel");
+    MARK(" + start show panel");
     gtk_widget_show (panel);
-    TIMER_ELAPSED(" + end show panel");
+    MARK(" + end show panel");
     g_idle_add ((GSourceFunc)expose_timeout, panel);
-    TIMER_ELAPSED("end panel_app_init_panel");
-}    
+    MARK("end panel_app_init_panel");
+}
 
 /**
  * panel_app_run
- * 
+ *
  * Run the panel application. Reads the configuration file(s) and sets up the
  * panels, before turning over control to the main event loop.
  *
- * Returns: 1 to restart and 0 to quit.
+ * Returns: RUN_FAILURE (2) if something goes wrong, RUN_RESTART (1) to restart 
+ *          and RUN_SUCCESS (0) to quit.
  **/
-int
-panel_app_run (int    argc, 
-               char **argv)
+gint
+panel_app_run (gchar *client_id)
 {
-#ifdef HAVE_SIGACTION
-    struct sigaction  act;
-#endif    
-    GIOChannel       *g_signal_in;
-    GError           *error = NULL;
-    long              fd_flags;
+    gchar      **restart_command;
 
-    /* create pipe and set writing end in non-blocking mode */
-    if (pipe (signal_pipe))
+    /* initialize signal handling */
+    if (!init_signal_pipe ())
     {
-        perror ("pipe");
-        return -1;
+        g_critical ("Unable to create signal-watch pipe: %s.", 
+                    strerror(errno));
+        return RUN_FAILURE;
     }
-    fd_flags = fcntl (signal_pipe[1], F_GETFL);
-    if (fd_flags == -1)
-    {
-        perror ("read descriptor flags");
-        return -1;
-    }
-    if (fcntl (signal_pipe[1], F_SETFL, fd_flags | O_NONBLOCK) == -1)
-    {
-        perror ("write descriptor flags");
-        return -1;
-    }
-
-    /* convert the reading end of the pipe into a GIOChannel */
-    g_signal_in = g_io_channel_unix_new (signal_pipe[0]);
-    g_io_channel_set_encoding (g_signal_in, NULL, &error);
-    if (error != NULL)
-    {                           /* handle potential errors */
-        g_printerr ("g_io_channel_set_encoding failed %s\n",
-                    error->message);
-        g_error_free (error);
-        return -1;
-    }
-
-    /* put the reading end also into non-blocking mode */
-    g_io_channel_set_flags (g_signal_in,
-                            g_io_channel_get_flags (g_signal_in) |
-                            G_IO_FLAG_NONBLOCK, &error);
-    if (error != NULL)
-    {
-        g_printerr ("g_io_set_flags failed %s\n", error->message);
-        g_error_free (error);
-        return -1;
-    }
-
-    /* register the reading end with the event loop */
-    g_io_add_watch (g_signal_in, G_IO_IN | G_IO_PRI, set_run_state, NULL);
-
-    /* register signals */
-#ifdef HAVE_SIGACTION
-    act.sa_handler = sighandler;
-    sigemptyset (&act.sa_mask);
-#ifdef SA_RESTART
-    act.sa_flags = SA_RESTART;
-#else
-    act.sa_flags = 0;
-#endif
-    sigaction (SIGHUP,  &act, NULL);
-    sigaction (SIGUSR1, &act, NULL);
-    sigaction (SIGUSR2, &act, NULL);
-    sigaction (SIGINT,  &act, NULL);
-    sigaction (SIGABRT, &act, NULL);
-    sigaction (SIGTERM, &act, NULL);
-    act.sa_handler = sigchld_handler;
-    sigaction (SIGCHLD, &act, NULL);
-#else
-    signal (SIGHUP,  sighandler);
-    signal (SIGUSR1, sighandler);
-    signal (SIGUSR2, sighandler);
-    signal (SIGINT,  sighandler);
-    signal (SIGABRT, sighandler);
-    signal (SIGTERM, sighandler);
-    signal (SIGCHLD, sigchld_handler);
-#endif
+    init_signal_handlers ();
 
     /* environment */
-    xfce_setenv ("DISPLAY", gdk_display_get_name (gdk_display_get_default ()),
-		 TRUE);
-    
+    xfce_setenv ("DISPLAY", 
+                 gdk_display_get_name (gdk_display_get_default ()),
+                 TRUE);
+
     /* session management */
+    restart_command    = g_new (gchar *, 3);
+    restart_command[0] = g_strdup ("xfce4-panel");
+    restart_command[1] = g_strdup ("-r");
+    restart_command[2] = NULL;
+
     panel_app.session_client = 
-        client_session_new (argc, argv, NULL /* data */, 
-                            SESSION_RESTART_IF_RUNNING, 40);
-
+        client_session_new_full (NULL, 
+                                 SESSION_RESTART_IMMEDIATELY, 
+                                 40, 
+                                 client_id,
+                                 PACKAGE_NAME,
+                                 NULL,
+                                 restart_command, 
+                                 g_strdupv (restart_command),
+                                 NULL,
+                                 NULL,
+                                 NULL);
     panel_app.session_client->save_yourself = session_save_yourself;
-    panel_app.session_client->die           = session_die;
+    panel_app.session_client->die = session_die;
 
-    TIMER_ELAPSED("connect to session manager");
+    MARK("connect to session manager");
     if (!session_init (panel_app.session_client))
     {
-        g_free (panel_app.session_client);
+        /* no indeed we're not connected... */
+        panel_app.session_client->current_state = SESSION_CLIENT_DISCONNECTED;
+        
+        /* cleanup */
+        client_session_free (panel_app.session_client);
         panel_app.session_client = NULL;
-    }   
-    
+    }
+
     /* screen layout and geometry */
-    TIMER_ELAPSED("start monitor list creation");
+    MARK("start monitor list creation");
     create_monitor_list ();
-    
+
     /* configuration */
-    TIMER_ELAPSED("start init item manager");
+    MARK("start init item manager");
     xfce_panel_item_manager_init ();
 
-    TIMER_ELAPSED("start panel creation");
+    MARK("start panel creation");
     panel_app.panel_list = panel_config_create_panels ();
-    TIMER_ELAPSED("end panel creation");
+    MARK("end panel creation");
 
-    g_ptr_array_foreach (panel_app.panel_list, (GFunc)panel_app_init_panel, 
+    g_ptr_array_foreach (panel_app.panel_list, (GFunc)panel_app_init_panel,
                          NULL);
 
     /* Run Forrest, Run! */
     panel_app.runstate = PANEL_RUN_STATE_NORMAL;
-    TIMER_ELAPSED("start main loop");
+    MARK("start main loop");
     gtk_main ();
-    
+    MARK("end main loop");
+
     /* cleanup */
-    g_free (panel_app.session_client);
+    if (panel_app.session_client)
+        client_session_free (panel_app.session_client);
 
     cleanup_panels ();
     xfce_panel_item_manager_cleanup ();
 
     if (panel_app.runstate == PANEL_RUN_STATE_RESTART)
-        return 1;
+        return RUN_RESTART;
 
-    return 0;
+    return RUN_SUCCESS;
 }
 
 static gboolean
 save_timeout (void)
 {
     DBG (" ++ save timeout");
-    
+
     if (panel_app.save_id)
         g_source_remove (panel_app.save_id);
     panel_app.save_id = 0;
@@ -768,7 +790,7 @@ save_timeout (void)
     return FALSE;
 }
 
-void 
+void
 panel_app_queue_save (void)
 {
     if (!panel_app.initialized)
@@ -777,26 +799,26 @@ panel_app_queue_save (void)
     if (panel_app.runstate == PANEL_RUN_STATE_NORMAL)
     {
         if (!panel_app.save_id)
-            panel_app.save_id = 
+            panel_app.save_id =
                 g_timeout_add (SAVE_TIMEOUT, (GSourceFunc)save_timeout, NULL);
     }
 }
 
-void 
+void
 panel_app_customize (void)
 {
     if (xfce_allow_panel_customization())
         panel_manager_dialog (panel_app.panel_list);
 }
 
-void 
+void
 panel_app_customize_items (GtkWidget *active_item)
 {
     if (xfce_allow_panel_customization())
         add_items_dialog (panel_app.panel_list, active_item);
 }
 
-void 
+void
 panel_app_save (void)
 {
     if (!panel_app.initialized)
@@ -806,35 +828,35 @@ panel_app_save (void)
         panel_config_save_panels (panel_app.panel_list);
 }
 
-void 
+void
 panel_app_restart (void)
 {
     panel_app.runstate = PANEL_RUN_STATE_RESTART;
-    check_run_state ();
+    evaluate_run_state ();
 }
 
-void 
+void
 panel_app_quit (void)
 {
     panel_app.runstate = PANEL_RUN_STATE_QUIT;
-    check_run_state ();
+    evaluate_run_state ();
 }
 
-void 
+void
 panel_app_quit_noconfirm (void)
 {
     panel_app.runstate = PANEL_RUN_STATE_QUIT_NOCONFIRM;
-    check_run_state ();
+    evaluate_run_state ();
 }
 
-void 
+void
 panel_app_quit_nosave (void)
 {
     panel_app.runstate = PANEL_RUN_STATE_QUIT_NOSAVE;
-    check_run_state ();
+    evaluate_run_state ();
 }
 
-void 
+void
 panel_app_add_panel (void)
 {
     Panel *panel;
@@ -846,7 +868,7 @@ panel_app_add_panel (void)
 
     if (G_UNLIKELY (panel_app.panel_list == NULL))
         panel_app.panel_list = g_ptr_array_sized_new (1);
-    
+
     g_ptr_array_add (panel_app.panel_list, panel);
 
     panel_set_screen_position (panel, XFCE_SCREEN_POSITION_FLOATING_H);
@@ -861,22 +883,22 @@ panel_app_add_panel (void)
     panel_app_customize ();
 }
 
-void 
+void
 panel_app_remove_panel (GtkWidget *panel)
 {
-    int   response = GTK_RESPONSE_NONE;
-    int   n;
-    char *first;
+    gint   response = GTK_RESPONSE_NONE;
+    guint  n;
+    gchar *first;
 
     if (!xfce_allow_panel_customization())
         return;
-        
+
     if (panel_app.panel_list->len == 1)
     {
-        response = 
+        response =
             xfce_message_dialog (NULL, _("Xfce Panel"),
-                                 GTK_STOCK_DIALOG_WARNING, 
-                                 _("Exit Xfce Panel?"), 
+                                 GTK_STOCK_DIALOG_WARNING,
+                                 _("Exit Xfce Panel?"),
                                  _("You can't remove the last panel. "
                                    "Would you like to exit the program?"),
                                  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -888,7 +910,7 @@ panel_app_remove_panel (GtkWidget *panel)
 
         return;
     }
-    
+
     for (n = 0; n < panel_app.panel_list->len; ++n)
     {
         if (panel == g_ptr_array_index (panel_app.panel_list, n))
@@ -897,24 +919,24 @@ panel_app_remove_panel (GtkWidget *panel)
 
     if (n == panel_app.panel_list->len)
         return;
-    
+
     panel_block_autohide (PANEL (panel));
-    
+
     panel_set_items_sensitive (PANEL (panel), FALSE);
     gtk_widget_set_sensitive (panel, FALSE);
     gtk_drag_highlight (panel);
-    
+
     first = g_strdup_printf (_("Remove Panel \"%d\"?"), n + 1);
-    
+
     response = xfce_message_dialog (GTK_WINDOW (panel), _("Xfce Panel"),
-                                    GTK_STOCK_DIALOG_WARNING, first, 
+                                    GTK_STOCK_DIALOG_WARNING, first,
                                     _("The selected panel and all its items "
                                       "will be removed."),
                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                     GTK_STOCK_REMOVE, GTK_RESPONSE_ACCEPT,
                                     NULL);
     g_free (first);
-                                    
+
     if (response != GTK_RESPONSE_ACCEPT)
     {
         gtk_drag_unhighlight (panel);
@@ -932,36 +954,34 @@ panel_app_remove_panel (GtkWidget *panel)
     }
 }
 
-void 
+void
 panel_app_about (GtkWidget *panel)
 {
     XfceAboutInfo *info;
     GtkWidget     *dlg;
     GdkPixbuf     *pb;
 
-    info = 
-        xfce_about_info_new (_("Xfce Panel"), "", _("Xfce Panel"), 
-                             XFCE_COPYRIGHT_TEXT ("2006", "Jasper Huijsmans"),
-                             XFCE_LICENSE_GPL);
+    info = xfce_about_info_new (_("Xfce Panel"), "", _("Xfce Panel"),
+                                XFCE_COPYRIGHT_TEXT ("2006", "Jasper Huijsmans"),
+                                XFCE_LICENSE_GPL);
 
     xfce_about_info_set_homepage (info, "http://www.xfce.org");
 
-    xfce_about_info_add_credit (info, "Jasper Huijsmans", "jasper@xfce.org",
-                                _("Developer"));
-
-    xfce_about_info_add_credit (info, "Nick Schermer", "nick@xfce.org",
-                                _("Developer"));
+    xfce_about_info_add_credit (info, "Jasper Huijsmans", "jasper@xfce.org", _("Developer"));
+    xfce_about_info_add_credit (info, "Nick Schermer", "nick@xfce.org", _("Developer"));
 
     pb = xfce_themed_icon_load ("xfce4-panel", 48);
     dlg = xfce_about_dialog_new_with_values (NULL, info, pb);
     g_object_unref (G_OBJECT (pb));
 
     gtk_window_set_screen (GTK_WINDOW (dlg), gtk_widget_get_screen (panel));
+
     gtk_widget_set_size_request (dlg, 400, 300);
 
     gtk_dialog_run (GTK_DIALOG (dlg));
 
     gtk_widget_destroy (dlg);
+
     xfce_about_info_free (info);
 }
 
@@ -974,37 +994,38 @@ panel_app_get_ipc_window (void)
 }
 
 XfceMonitor *
-panel_app_get_monitor (int n)
+panel_app_get_monitor (guint n)
 {
-    return g_ptr_array_index (panel_app.monitor_list, 
-                              CLAMP (n, 0, panel_app.monitor_list->len - 1));
+    return g_ptr_array_index (panel_app.monitor_list,
+                              MIN (n, panel_app.monitor_list->len - 1));
 }
 
-int
+guint
 panel_app_get_n_monitors (void)
 {
     return panel_app.monitor_list->len;
 }
 
 /* open dialogs */
-void 
+void
 panel_app_register_dialog (GtkWidget *dialog)
 {
     g_return_if_fail (GTK_IS_WIDGET (dialog));
 
-    g_signal_connect (dialog, "destroy", G_CALLBACK (unregister_dialog), NULL);
+    g_signal_connect (G_OBJECT (dialog), "destroy",
+                      G_CALLBACK (unregister_dialog), NULL);
 
     panel_app.dialogs = g_list_prepend (panel_app.dialogs, dialog);
 }
 
 /* current panel */
-void 
+void
 panel_app_set_current_panel (gpointer *panel)
 {
-    int i;
+    guint i;
 
     panel_app.current_panel = 0;
-    
+
     for (i = 0; i < panel_app.panel_list->len; ++i)
     {
         if (g_ptr_array_index (panel_app.panel_list, i) == panel)
@@ -1017,16 +1038,16 @@ panel_app_set_current_panel (gpointer *panel)
     DBG ("Current panel: %d", panel_app.current_panel);
 }
 
-void 
+void
 panel_app_unset_current_panel (gpointer *panel)
 {
-    int i;
+    guint i;
 
     for (i = 0; i < panel_app.panel_list->len; ++i)
     {
         if (g_ptr_array_index (panel_app.panel_list, i) == panel)
         {
-            if (i == panel_app.current_panel)
+            if (i == (guint) panel_app.current_panel)
                 panel_app.current_panel = 0;
             break;
         }
@@ -1035,13 +1056,13 @@ panel_app_unset_current_panel (gpointer *panel)
     DBG ("Current panel: %d", panel_app.current_panel);
 }
 
-int 
+gint
 panel_app_get_current_panel (void)
 {
     return panel_app.current_panel;
 }
 
-G_CONST_RETURN GPtrArray *
+const GPtrArray *
 panel_app_get_panel_list (void)
 {
     return panel_app.panel_list;
